#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Horse Racing Auto
Version: 1.0 20130207
Codename:C.Y.F.L. 
Author: DarkRanger <darkranger.no-ip.org>
License: GNU General Public License
History:
* 02/2011 - Finished by speed coding.
* 01/2012 - Slightly modified for Nokia N900.
* 02/2013 - Massive rewrite and clean up.
'''
import os, sys
import logging
import pygame
import random
import time

from pygame.locals import *

PRODUCT_NAME = "Horse Racing Auto"
IMAGE_RIDER = "images/horse_rider.bmp"
IMAGE_WIN = "images/horse_win.bmp"
FONT_ITALIC = "fonts/DejaVuSans-BoldOblique.ttf"
FONT_BOLD = "fonts/DejaVuSans-Bold.ttf"
LOG_FILE = "record.log"
TIMER = 30
WIDTH = 800
HEIGHT = 450
END_LINE = 40
POINTER = 80
FULL_SCREEN = 1
LOOPS = 0

class Horse:
    def __init__(self, number, spritePos, gamePos):
        self.image = pygame.image.load(internalResource(IMAGE_RIDER)).convert()
        self.spriteArea = pygame.Rect(0, spritePos, 80, 60)
        self.colorkey = self.image.get_at((0,0))
        self.image.set_colorkey(self.colorkey, RLEACCEL)
        self.rect = self.image.get_rect()
        self.rect.topleft = (WIDTH / 2 - END_LINE, gamePos)
        self.forward = random.randint(0, WIDTH - END_LINE)
        self.backward = random.randint(END_LINE, WIDTH / 2)
        self.speed = [0, 0]
        self.number = number
        
    def move(self):
        self.rect = self.rect.move(self.speed)
        if self.rect.left <=  self.backward or self.rect.right >= self.forward :
            if random.randint(1,10) % 2 == 1:
                self.speed[0] = -self.speed[0]
            else: 
                self.forward = random.randint(END_LINE, WIDTH + (WIDTH /3))
                self.backward = random.randint(END_LINE, WIDTH)
                
    def score(self, place):
        self.place = place
        
    def result(self, screen):
        font = pygame.font.Font(internalResource(FONT_BOLD), 23)
        if self.place == 1:
           endwith = "st"
        if self.place == 2:
           endwith = "nd" 
        if self.place == 3:
           endwith = "rd" 
        if self.place == 4:
           endwith = "th"  
        if self.place == 5:
           endwith = "th"     
        if self.place == 1: 
            text = font.render("%d%s" % (self.place, endwith), True, (255,0,0), (0, 0, 0))        
        else:        
            text = font.render("%d%s" % (self.place, endwith), True, (255,255,255), (0, 0, 0))
        textRect = text.get_rect()
        textRect.x = self.rect.x + 100
        textRect.y = self.rect.y
        screen.blit(text, textRect)
        
        
class Wins:
    def __init__(self, number):
        if number == 1:
            spritePos = 1220
        if number == 2:
            spritePos = 620
        if number == 3:
            spritePos = 320
        if number == 4:
            spritePos = 920         
        if number == 5:
            spritePos = 20
        self.image = pygame.image.load(internalResource(IMAGE_WIN)).convert()
        self.spriteArea = pygame.Rect(40, spritePos , 300, 270)
        self.image.set_colorkey(None, RLEACCEL)
        self.rect = self.image.get_rect()
        self.rect.topleft = (WIDTH / 4 + 60, HEIGHT / 6)
        self.number = number    
     
     
def doLogging(msg):
    logging.basicConfig(filename=externalResource(LOG_FILE), level=logging.DEBUG)
    logging.info(msg) 
    print msg    
                
def internalResource(path):
    if hasattr(sys, "_MEIPASS"):
        return os.path.abspath(os.path.join(sys._MEIPASS,path))
    else:
        return os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), path))
      
def externalResource(path):
    return os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), path))
    
def makeHorses(number):
    horses = []
    spritePos = 0
    gamePos = 10
    for x in range(1, number + 1):
        horses.append(Horse(x, spritePos, gamePos))
        spritePos = spritePos + 60
        gamePos = gamePos + 70
    return horses
    
def makeWins(number):
    wins = []
    for x in range(1, number + 1):
        wins.append(Wins(x))
    return wins
    
def moveHorses(horses):
    for x in range(0, len(horses)): 
        horses[x].move()

def spriteHorses(screen, running, pointer, horses):
    spritePos = 0
    if running == True:
        for x in range(0, len(horses)): 
            horses[x].spriteArea = pygame.Rect(pointer, spritePos, 80, 60)
            spritePos = spritePos + 60
        pointer = pointer + 84
        if pointer >= 480:
            pointer = 0 
    drawHorses(screen, horses)        
    return pointer  

def drawHorses(screen, horses):
    for x in range(0, len(horses)):
        screen.blit(horses[x].image, horses[x].rect, horses[x].spriteArea)

def drawWins(screen, horses, wins, gameResult):
    for x in range(0, len(horses)):  
        if horses[x].place == 1:    
            screen.blit(wins[x].image, wins[x].rect, wins[x].spriteArea)
            pygame.display.update()
            pygame.time.delay(1500)
    return gameResult + 1

def startingHorse(horses):
    pos = 10
    for x in range(0, len(horses)):
        horses[x].rect.topleft = (WIDTH / 2 - END_LINE, pos)
        pos = pos + 70   

def speedHorses(horses):
    for x in range(0, len(horses)):
        if random.randint(1,10) % 2 == 0:
            horses[x].speed = [2, 0]
        else:
            horses[x].speed = [-2, 0]    

def scoreHorses(horses):
    place = []
    placeClean = []
    for x in range(0, len(horses)):
        place.append(horses[x].rect.right)
    for item in set(place):
        placeClean.append(item)
    placeClean.sort(reverse = True)      
    for x in range(0, len(horses)):
        for y in range(0, len(placeClean)):
            if horses[x].rect.right == placeClean[y]:
                doLogging("Horse %d: score: %d, place: %d" % (x + 1, horses[x].rect.right, y + 1))
                horses[x].score(y + 1)
                break

def placeHorses(screen, horses):
    for x in range(0, len(horses)):
        horses[x].result(screen)  
        
def musicPlayback(key):
    if key == K_1 or key == K_KP1:
        if os.path.exists(externalResource("music/1.ogg")):
           pygame.mixer.music.load("music/1.ogg")
           pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_2 or key == K_KP2:                           
        if os.path.exists(externalResource("music/2.ogg")):
            pygame.mixer.music.load("music/2.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_3 or key == K_KP3:                           
        if os.path.exists(externalResource("music/3.ogg")):
            pygame.mixer.music.load("music/3.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_4 or key == K_KP4:                           
        if os.path.exists(externalResource("music/4.ogg")):
            pygame.mixer.music.load("music/4.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_5 or key == K_KP5:                           
        if os.path.exists(externalResource("music/5.ogg")):
            pygame.mixer.music.load("music/5.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_6 or key == K_KP6:                           
        if os.path.exists(externalResource("music/6.ogg")):
            pygame.mixer.music.load("music/6.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_7 or key == K_KP7:                           
        if os.path.exists(externalResource("music/7.ogg")):
            pygame.mixer.music.load("music/7.ogg")
            pygame.mixer.music.play(LOOPS, 0.0)
    if key == K_8 or key == K_KP8:       
        pygame.mixer.music.pause()
    if key == K_9 or key == K_KP9:       
        pygame.mixer.music.unpause()                        
    if key == K_0 or key == K_KP0:                
        pygame.mixer.music.stop()  

def drawsLines(screen, number):
    linePos = 75
    for x in range(0, number):
        pygame.draw.line(screen, (255, 255, 255), (0, linePos), (WIDTH, linePos), 4)
        linePos = linePos + 75
        
def drawsNumbers(screen, number):
    numberPos = 45
    font = pygame.font.Font(internalResource(FONT_BOLD), 17)
    for x in range(0, number):
        text = font.render("%s" % str(x + 1), True, (255,255,255), (55,200,0))
        textRect = text.get_rect()
        textRect.x = WIDTH - 30
        textRect.y = numberPos
        screen.blit(text, textRect)
        numberPos = numberPos + 75
        
def cyfl(appdir):
    pygame.init()
    runway = WIDTH
    pointer = POINTER
    game = 0
    gameResult = 0

    if FULL_SCREEN == 0:
        screen = pygame.display.set_mode((WIDTH, HEIGHT))    
    else:
        screen = pygame.display.set_mode((WIDTH, HEIGHT), FULLSCREEN)
        
    pygame.display.set_caption(PRODUCT_NAME)
    pygame.mouse.set_visible(1)
    
    font = pygame.font.Font(internalResource(FONT_ITALIC), 17)
    title = font.render(PRODUCT_NAME, True, (255,255,255), (0, 0, 0))
    titleRect = title.get_rect()
    titleRect.x = WIDTH / 2 - 80
    titleRect.y = HEIGHT - 90
    info = font.render("Use left button of mouse or SPACE key to start", True, (255,255,255), (0, 0, 0))
    infoRect = info.get_rect()
    infoRect.x = WIDTH / 2 - 220
    infoRect.y = HEIGHT - 70
    note = font.render("Number key 1-6 for music playback, 8,9,0 for pause, unpause and stop", True, (255,255,255), (0, 0, 0))
    noteRect = note.get_rect()
    noteRect.x = WIDTH / 2 - 340
    noteRect.y = HEIGHT - 50
    
    fontBig = pygame.font.Font(internalResource(FONT_BOLD), 50)    
    clock = fontBig.render("", True, (255,255,255), (0, 0, 0))
    clockRect = clock.get_rect()
    clockRect.x = WIDTH / 2 - 60
    clockRect.y = HEIGHT - 80
    
    horses = makeHorses(5)
    wins = makeWins(5)
    starting = time.time()
    running = False

    while 1:
          while running == True:
              for event in pygame.event.get():
                if event.type == pygame.QUIT: 
                    sys.exit()
                if event.type == KEYDOWN:
                    musicPlayback(event.key)                      
                    if event.key == K_ESCAPE:
                        sys.exit()
                       
              if time.time() - starting >= TIMER:
                  running = False
                  scoreHorses(horses)
                  count = "00.0"
                  clock = fontBig.render(count, True, (255,0,0), (0, 0, 0))
              else:
                  remaining = (TIMER - (time.time() - starting))
                  count = "%04.1f" % remaining
                  if remaining <= 8:
                      clock = fontBig.render(count, True, (255,0,0), (0, 0, 0))
                  else:
                      clock = fontBig.render(count, True, (255,255,255), (0, 0, 0))
                  moveHorses(horses)

              pygame.time.delay(80)
              screen.fill((55, 200, 0))
              drawsLines(screen, 5)
              runway = runway - 60
              pygame.draw.line(screen, (255, 255, 255), (0 + runway, 0), (0 + runway, HEIGHT), 4)
              if runway <= 0:
                  runway = WIDTH
    
              pointer = spriteHorses(screen, running, pointer, horses)
              screen.blit(clock, clockRect)
              pygame.display.update()
              
          screen.fill((55, 200, 0))
          drawsLines(screen, 5)
          drawsNumbers(screen, 5)
          pygame.draw.line(screen, (255, 255, 255), (0 + runway, 0), (0 + runway, HEIGHT), 4)
          drawHorses(screen, horses)
          
          if game >= 1:
              if gameResult < game:
                  gameResult = drawWins(screen, horses, wins, gameResult)          
              placeHorses(screen, horses)
              screen.blit(clock, clockRect)
              drawsNumbers(screen, 5)
              if gameResult < game:
                  gameResult = drawWins(screen, horses, wins, gameResult)
          else:
              screen.blit(title, titleRect)
              screen.blit(info, infoRect)
              screen.blit(note, noteRect)  
          
          pygame.display.update()
          
          for event in pygame.event.get():
              if event.type == pygame.QUIT: 
                  sys.exit()
              if event.type == KEYDOWN:
                   musicPlayback(event.key)                                         
                   if event.key == K_ESCAPE:
                       sys.exit()        
                   if event.key == K_SPACE and running == False:
                       running = True               
              if event.type == MOUSEBUTTONDOWN:                                       
                  if event.button == 1 and running == False:
                      running = True

          if running == True: 
              starting = time.time()
              startingHorse(horses)
              game = game + 1
              doLogging("Round %d" % game)
              speedHorses(horses)              
                    
if __name__ == "__main__":
    appdir = os.path.abspath(os.path.dirname(sys.argv[0]))
    os.chdir(appdir)
    if os.path.exists(externalResource(LOG_FILE)):
        os.remove(LOG_FILE)
    cyfl(appdir)
